import { z } from 'zod';
import { FIRESTORE_API_GET_LIMIT } from '../../constants';

// for 'get' query params, .or shenanigans
export const firestoreGetLimitParser = z
  .number()
  .int()
  .or(z.string().regex(/^\d+$/).transform(Number))
  .refine((n) => n >= 0, {
    message: 'limit must be a positive number',
  })
  .refine((n) => n <= FIRESTORE_API_GET_LIMIT, {
    message: `limit must be less than ${FIRESTORE_API_GET_LIMIT}`,
  });
// for 'get' query params, .or shenanigans
export const firestoreGetOffsetParser = z
  .number()
  .int()
  .or(z.string().regex(/^\d+$/).transform(Number))
  .refine((n) => n >= 0, {
    message: 'offset must be a positive number',
  });
export const firestoreListLimitParamsParser = z.object({
  limit: firestoreGetLimitParser.optional().default(FIRESTORE_API_GET_LIMIT),
});
export const firestoreListOffsetParamsParser = z.object({
  offset: firestoreGetOffsetParser.optional().default(0),
});

export const firestoreListParamsParser = firestoreListLimitParamsParser.merge(firestoreListOffsetParamsParser);
