import { z } from 'zod';
import { PayinMethodOriginId } from './types';
import { stripePaymentMethodIdParser } from '../stripe/parsers';
import { PAYIN_METHOD_ORIGIN_TYPES } from './constants';
import { PAYIN_METHOD_GET_ORIGIN_ID_PARAM_NAME, PAYIN_METHOD_GET_ORIGIN_TYPE_PARAM_NAME } from './route.constants';
import { payinProviderCustomerIdParser } from '../payinProvider/parsers';

export const payinMethodOriginIdParser = stripePaymentMethodIdParser /*.or(paypal...)*/
  .refine((o) => {
    const r: PayinMethodOriginId = o; // assert
    return r;
  });

export const payinMethodOriginTypeParser = z.enum(PAYIN_METHOD_ORIGIN_TYPES);

export const stripePayinMethodOriginIdParser = z.object({
  [PAYIN_METHOD_GET_ORIGIN_ID_PARAM_NAME]: stripePaymentMethodIdParser,
  [PAYIN_METHOD_GET_ORIGIN_TYPE_PARAM_NAME]: z.literal('stripe'),
});
export const paypalPayinMethodOriginIdParser = z.object({
  [PAYIN_METHOD_GET_ORIGIN_ID_PARAM_NAME]: z.string(), // TODO
  [PAYIN_METHOD_GET_ORIGIN_TYPE_PARAM_NAME]: z.literal('paypal'),
});
export const payinMethodUpsertParamsParser = z.discriminatedUnion(PAYIN_METHOD_GET_ORIGIN_TYPE_PARAM_NAME, [
  stripePayinMethodOriginIdParser,
  paypalPayinMethodOriginIdParser,
]);
export const payinMethodUpsertResultParser = payinProviderCustomerIdParser;
